"""Sphinx configuration."""

from datetime import datetime

project = "deepbiop"
author = "Yangyang Li"
copyright = f"{datetime.now().year}, {author}"
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.napoleon",
    "sphinx.ext.intersphinx",
    "myst_parser",
    "autoapi.extension",
]
source_suffix = [".rst", ".md"]
autodoc_typehints = "description"
html_theme = "furo"
html_title = "DeepBioP"
language = "en"
autoapi_dirs = ["../deepbiop"]

autoapi_add_toctree_entry = True

myst_heading_anchors = 3
myst_enable_extensions = [
    "dollarmath",
    "amsmath",
    "deflist",
    "fieldlist",
    "html_admonition",
    "html_image",
    "colon_fence",
    "smartquotes",
    "replacements",
    "linkify",
    "strikethrough",
    "substitution",
    "tasklist",
]


extlinks = {
    "pypi": ("https://pypi.org/project/%s/", "%s"),
}


intersphinx_mapping = {
    "python": ("https://docs.python.org/3", None),
    "sphinx": ("https://www.sphinx-doc.org/en/master", None),
    "numpy": ("https://numpy.org/doc/stable/", None),
    "pandas": ("https://pandas.pydata.org/docs/", None),
    "pyarrow": ("https://arrow.apache.org/docs/", None),
}


html_theme_options = {
    "footer_icons": [
        {
            "name": "GitHub",
            "url": "https://github.com/cauliyang/DeepBioP",
            "html": """
                <svg stroke="currentColor" fill="currentColor" stroke-width="0" viewBox="0 0 16 16">
                    <path fill-rule="evenodd" d="M8 0C3.58 0 0 3.58 0 8c0 3.54 2.29 6.53 5.47 7.59.4.07.55-.17.55-.38 0-.19-.01-.82-.01-1.49-2.01.37-2.53-.49-2.69-.94-.09-.23-.48-.94-.82-1.13-.28-.15-.68-.52-.01-.53.63-.01 1.08.58 1.23.82.72 1.21 1.87.87 2.33.66.07-.52.28-.87.51-1.07-1.78-.2-3.64-.89-3.64-3.95 0-.87.31-1.59.82-2.15-.08-.2-.36-1.02.08-2.12 0 0 .67-.21 2.2.82.64-.18 1.32-.27 2-.27.68 0 1.36.09 2 .27 1.53-1.04 2.2-.82 2.2-.82.44 1.1.16 1.92.08 2.12.51.56.82 1.27.82 2.15 0 3.07-1.87 3.75-3.65 3.95.29.25.54.73.54 1.48 0 1.07-.01 1.93-.01 2.2 0 .21.15.46.55.38A8.013 8.013 0 0 0 16 8c0-4.42-3.58-8-8-8z"></path>
                </svg>
            """,
            "class": "",
        },
    ],
}



from deepbiop import utils


def test_hight_targets():
    result = utils.highlight_targets("ACTGAACCGAGATCGAGTG", [(0, 3), (7, 10)])
    print(result)


# def test_segment():
#     print(utils.__all__)

#     segment = utils.GenomicInterval("chr10", 300, 5000)
#     segment2 = utils.GenomicInterval("chr10", 300, 5000)

#     assert segment.overlap(segment2)

#     print(segment)


def test_reverse_complement():
    result = utils.reverse_complement("ACTGAACCGAGATCGAGTG")
    print(result)



# This file is automatically generated by pyo3_stub_gen
# ruff: noqa: E501, F401

import os
import pathlib
import typing

class GenomicInterval:
    r"""
    A segment is a genomic interval defined by a chromosome, a start position and an end position.
    The start position is inclusive and the end position is exclusive.
    """

    start: int
    end: int
    chr: str
    def __new__(cls, chr: str, start: int, end: int): ...
    def set_chr(self, chr: str) -> None: ...
    def overlap(self, other: GenomicInterval) -> bool: ...
    def __repr__(self) -> str: ...

class PslAlignment:
    qname: str
    qsize: int
    qstart: int
    qend: int
    qmatch: int
    tname: str
    tsize: int
    tstart: int
    tend: int
    identity: float
    def __repr__(self) -> str: ...

def generate_unmaped_intervals(
    input: typing.Sequence[tuple[int, int]], total_length: int
) -> list[tuple[int, int]]: ...
def highlight_targets(
    sequence: str, targets: typing.Sequence[tuple[int, int]], text_width: int | None
) -> str: ...
def majority_voting(labels: typing.Sequence[int], window_size: int) -> list[int]: ...
def parse_psl_by_qname(
    file_path: str | os.PathLike | pathlib.Path,
) -> dict[str, list[PslAlignment]]:
    r"""Parse PSL file by query name."""

def remove_intervals_and_keep_left(
    seq: str, intervals: typing.Sequence[tuple[int, int]]
) -> tuple[list[str], list[tuple[int, int]]]: ...
def reverse_complement(seq: str) -> str: ...



# This file is automatically generated by pyo3_stub_gen
# ruff: noqa: E501, F401

import os
import pathlib
import typing

def count_chimeric_reads_for_path(
    bam: str | os.PathLike | pathlib.Path, threads: int | None
) -> int:
    r"""Calculate the number of chimeric reads in a BAM file."""

def count_chimeric_reads_for_paths(
    bams: typing.Sequence[str | os.PathLike | pathlib.Path], threads: int | None
) -> dict[str, int]:
    r"""Calculate the number of chimeric reads in multiple BAM files."""

def left_right_soft_clip(cigar_string: str) -> tuple[int, int]:
    r"""Calculate left and right soft clips from a cigar string."""



# This file is automatically generated by pyo3_stub_gen
# ruff: noqa: E501, F401

import os
import pathlib
import typing

import numpy
import numpy.typing

class FqEncoderOption:
    kmer_size: int
    qual_offset: int
    bases: list[int]
    vectorized_target: bool
    threads: int
    def __new__(
        cls,
        kmer_size: int,
        qual_offset: int,
        bases: str,
        vectorized_target: bool,
        threads: int | None,
    ): ...

class JsonEncoder:
    def __new__(cls, option: FqEncoderOption): ...

class ParquetEncoder:
    def __new__(cls, option: FqEncoderOption): ...

class Predict:
    r"""A struct to store the prediction result."""

    prediction: list[int]
    seq: str
    id: str
    is_truncated: bool
    qual: str | None
    def __new__(
        cls,
        prediction: typing.Sequence[int],
        seq: str,
        id: str,
        is_truncated: bool,
        qual: str | None,
    ): ...
    def __repr__(self) -> str: ...
    def prediction_region(self) -> list[tuple[int, int]]:
        r"""Get the prediction region."""

    def smooth_prediction(self, window_size: int) -> list[tuple[int, int]]:
        r"""Get the smooth prediction region."""

    def smooth_label(self, window_size: int) -> list[int]:
        r"""Get the smooth label."""

    def smooth_and_select_intervals(
        self,
        smooth_window_size: int,
        min_interval_size: int,
        append_interval_number: int,
    ) -> list[tuple[int, int]]:
        r"""Smooth and select intervals."""

    def seq_len(self) -> int:
        r"""Get the sequence length."""

    def qual_array(self) -> list[int]:
        r"""Get the quality score array."""

    def show_info(
        self, smooth_interval: typing.Sequence[tuple[int, int]], text_width: int | None
    ) -> str:
        r"""Show the information of the prediction."""

    def __getstate__(self) -> typing.Any: ...
    def __setstate__(self, state: typing.Any) -> None: ...

class RecordData:
    id: str
    seq: str
    qual: str
    def __new__(cls, id: str, seq: str, qual: str): ...
    def set_id(self, id: str) -> None: ...
    def set_seq(self, seq: str) -> None: ...
    def set_qual(self, qual: str) -> None: ...

def convert_multiple_fqs_to_one_fq(
    paths: typing.Sequence[str | os.PathLike | pathlib.Path],
    result_path: str | os.PathLike | pathlib.Path,
    parallel: bool,
) -> None: ...
def encode_fq_path_to_json(
    fq_path: str | os.PathLike | pathlib.Path,
    k: int,
    bases: str,
    qual_offset: int,
    vectorized_target: bool,
    result_path: str | os.PathLike | pathlib.Path | None,
) -> None: ...
def encode_fq_path_to_parquet(
    fq_path: str | os.PathLike | pathlib.Path,
    bases: str,
    qual_offset: int,
    vectorized_target: bool,
    result_path: str | os.PathLike | pathlib.Path | None,
) -> None: ...
def encode_fq_path_to_parquet_chunk(
    fq_path: str | os.PathLike | pathlib.Path,
    chunk_size: int,
    parallel: bool,
    bases: str,
    qual_offset: int,
    vectorized_target: bool,
) -> None: ...
def encode_fq_path_to_tensor(
    fq_path: str | os.PathLike | pathlib.Path,
    k: int,
    bases: str,
    qual_offset: int,
    vectorized_target: bool,
    max_width: int | None,
    max_seq_len: int | None,
) -> tuple[
    numpy.typing.NDArray[numpy.int32],
    numpy.typing.NDArray[numpy.int32],
    numpy.typing.NDArray[numpy.int32],
    dict[str, int],
]: ...
def encode_fq_paths_to_parquet(
    fq_path: typing.Sequence[str | os.PathLike | pathlib.Path],
    bases: str,
    qual_offset: int,
    vectorized_target: bool,
) -> None: ...
def encode_fq_paths_to_tensor(
    fq_paths: typing.Sequence[str | os.PathLike | pathlib.Path],
    k: int,
    bases: str,
    qual_offset: int,
    vectorized_target: bool,
    parallel_for_files: bool,
    max_width: int | None,
    max_seq_len: int | None,
) -> tuple[
    numpy.typing.NDArray[numpy.int32],
    numpy.typing.NDArray[numpy.int32],
    numpy.typing.NDArray[numpy.int32],
    dict[str, int],
]: ...
def encode_qual(qual: str, qual_offset: int) -> list[int]:
    r"""Convert ASCII quality to Phred score for Phred+33 encoding."""

def fastq_to_fasta(
    fastq_path: str | os.PathLike | pathlib.Path,
    fasta_path: str | os.PathLike | pathlib.Path,
) -> None: ...
def generate_kmers(base: str, k: int) -> list[str]: ...
def generate_kmers_table(base: str, k: int) -> dict[list[int], int]: ...
def get_label_region(labels: typing.Sequence[int]) -> list[tuple[int, int]]: ...
def kmers_to_seq(kmers: typing.Sequence[str]) -> str: ...
def load_predicts_from_batch_pt(
    pt_path: str | os.PathLike | pathlib.Path,
    ignore_label: int,
    id_table: typing.Mapping[int, str],
) -> dict[str, Predict]: ...
def load_predicts_from_batch_pts(
    pt_path: str | os.PathLike | pathlib.Path,
    ignore_label: int,
    id_table: typing.Mapping[int, str],
    max_predicts: int | None,
) -> dict[str, Predict]: ...
def normalize_seq(seq: str, iupac: bool) -> str:
    r"""
    Normalize a DNA sequence by converting any non-standard nucleotides to standard ones.

    This function takes a DNA sequence as a `String` and a boolean flag `iupac` indicating whether to normalize using IUPAC ambiguity codes.
    It returns a normalized DNA sequence as a `String`.

    # Arguments

    * `seq` - A DNA sequence as a `String`.
    * `iupac` - A boolean flag indicating whether to normalize using IUPAC ambiguity codes.

    # Returns

    A normalized DNA sequence as a `String`.
    """

def seq_to_kmers(seq: str, k: int, overlap: bool) -> list[str]: ...
def test_predicts(predicts: typing.Sequence[Predict]) -> None: ...
def write_fq(
    records_data: typing.Sequence[RecordData],
    file_path: str | os.PathLike | pathlib.Path | None,
) -> None: ...
def write_fq_parallel(
    records_data: typing.Sequence[RecordData],
    file_path: str | os.PathLike | pathlib.Path,
    threads: int,
) -> None: ...
